#include <SPI.h>
#include <SD.h>
#include <Wire.h>
#include <DS3231.h>
#include <Ethernet.h>
#include <EEPROM.h>
#define number_of_74hc595s 1  
#define numOfRegisterPins number_of_74hc595s * 8
#define LOGFILE "datalog.txt"
#define REQ_BUF_SZ   50
#define aref_voltage 3.30

const int chipSelect = 4;    
DS3231 clock;
RTCDateTime dt;

int SER_Pin = 30;   //pin 14 on the 75HC595 
int RCLK_Pin = 29;  //pin 12 on the 75HC595 
int SRCLK_Pin = 28; //pin 11 on the 75HC595  
int tempw = A10;            // the tmp36 and 48K pulldown are connected to a10
int tempwReading;
int photocellPin = A11;     // the cell and 10K pulldown are connected to a11
int photocellReading;     // the analog reading from the sensor divider
int floter = A12;
int floterReading;

boolean registers[numOfRegisterPins]; 

File dataFile;
// MAC address from Ethernet shield sticker under board
byte mac[] = { 
  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 0, 20); // IP address, may need to change depending on network
EthernetServer server(80);  // create a server at port 80
File webFile;               // the web page file on the SD card
char HTTP_req[REQ_BUF_SZ] = {
  0}; // buffered HTTP request stored as null terminated string
char req_index = 0;              // index into HTTP_req buffer


void setup() {
  analogReference(EXTERNAL);

  // epanafora reley meta apo diakopi revmatos apo tin eeprom
  EEPROM.write(8, 1);
  setreley();

  pinMode(SER_Pin, OUTPUT); 
  pinMode(RCLK_Pin, OUTPUT); 
  pinMode(SRCLK_Pin, OUTPUT); 

  Serial.begin(9600); 
  Serial.println("Initializing datalogger with RTC version 1.0");
  Serial.println("Starting SDCard reader and card");
  // disable Ethernet chip
  pinMode(10, OUTPUT);
  digitalWrite(10, HIGH);
  pinMode(3, OUTPUT);
  // initialize SD card
  Serial.println("Initializing SD card...");
  if (!SD.begin(4)) {
    Serial.println("ERROR - SD card initialization failed!");
    return;    // init failed
  }

  Serial.println("SUCCESS - SD card initialized.");
  // check for datalog.txt file
  if (!SD.exists("datalog.txt")) {
    Serial.println("ERROR - Can't find datalog.txt file!");
    return;  // can't find datalog file
  }
  if (!SD.exists("index.htm")) {
    Serial.println("ERROR - Can't find index.htm file!");
    return;  // can't find index file
  }
  Serial.println("SUCCESS - Found datalog.txt file.");
  Serial.println("SUCCESS - Found index.htm file.");
  Serial.println("Initialize DS3231");

  clock.begin();
  clock.armAlarm2(false);
  clock.clearAlarm2();
  // Set Alarm2 - Every 59m in each hour
  // setAlarm2(Date or Day, Hour, Minute, Mode, Armed = true)
  clock.setAlarm2(0, 0, 5, DS3231_MATCH_M);
  checkAlarms();
  // Set sketch compiling time
  clock.setDateTime(__DATE__, __TIME__); 

  Ethernet.begin(mac, ip);  // initialize Ethernet device
  server.begin();           // start to listen for clients
}

void loop(){

  clock.forceConversion();
  floterReading = analogRead(floter);
  floterReading = map(floterReading, 0, 1023, 0, 60); // sto floter moy perno 500 gia ta 30 cm poy doylevi (theli rithmisi stin limni)
  //// temp tmp36 start
  tempwReading = analogRead(tempw);
  float voltage = tempwReading * aref_voltage;
  voltage /= 1024.0; 
  float tempCW = (voltage - 0.5) * 100 ;  //converting from 10 mv per degree wit 500 mV offset
  Serial.print(tempCW); 
  Serial.println(" water degrees C");  
  //// temp tmp36 stop
  /////photocell start
  photocellReading = analogRead(photocellPin);  
  Serial.print("Analog reading = ");
  Serial.println(photocellReading);     // the raw analog reading
  ///////photocell stop

  // Serial.print("Temperature: ");
  int temperature1 =(clock.readTemperature());
  int temperature2 = tempCW;
  dt = clock.getDateTime();
  char temp1[10];//2 int, 2 dec, 1 point, and \0
  char temp2[10];
  char flot[10];
  char phtcell1[10];
  char msg[16];
  String str;

  str=String(temperature1);
  str.toCharArray(temp1,10);
  str=String(temperature2);
  str.toCharArray(temp2,10);
  str=String(photocellReading);
  str.toCharArray(phtcell1,10);
  str=String(floterReading);
  str.toCharArray(flot,10);

  //Compile a comma delimited string to send to the log
  sprintf(msg,"%s,%s,%s", temp1, temp2, phtcell1, flot);

  // Call isAlarm1(false) if you want clear alarm1 flag manualy by clearAlarm1();
  if (clock.isAlarm2())
  {
    Serial.println("SD data log");
    Serial.println("ALARM 2 TRIGGERED!");
    logThis(msg); 
  }

  /////////wep1

  EthernetClient client = server.available();  // try to get client
  if (client) {  // got client?
    boolean currentLineIsBlank = true;
    String buffer = ""; 
    while (client.connected()) {
      if (client.available()) {   // client data available to read
        char c = client.read(); // read 1 byte (character) from client
        //        Serial.print(c);    
        buffer+=c;
        // buffer first part of HTTP request in HTTP_req array (string)
        // leave last element in array as 0 to null terminate string (REQ_BUF_SZ - 1)
        if (req_index < (REQ_BUF_SZ - 1)) {
          HTTP_req[req_index] = c;          // save HTTP request character
          req_index++;
        }
        // last line of client request is blank and ends with \n
        // respond to client only after last line received
        if (c == '\n' && currentLineIsBlank) {
          // send a standard http response header
          client.println("HTTP/1.1 200 OK");
          // remainder of header follows below, depending on if
          // web page or XML page is requested
          // Ajax request - send XML file
          if (StrContains(HTTP_req, "ajax_inputs")) {
            // send rest of HTTP header
            client.println("Content-Type: text/xml");
            client.println("Connection: keep-alive");
            client.println();
            // send XML file containing input states
            XML_response(client);
          }
          // test gia download    
          else if (StrContains(HTTP_req, "GET /datalog.txt")) {
            //// test
            client.println("Content-Type: text/plain");
            client.println("Connection: keep-alive");
            client.println();           
            
            ///test
            
            
            webFile = SD.open("datalog.txt");
            if (webFile) {
              while(webFile.available()) {
                client.write(webFile.read()); // send web page to client
              }
              webFile.close();
            }
          }
          //end test gia download
          else {  // web page request
            // send rest of HTTP header
            client.println("Content-Type: text/html");
            client.println("Connection: keep-alive");
            client.println();
            // send web page
            webFile = SD.open("index.htm");        // open web page file
            if (webFile) {
              while(webFile.available()) {
                client.write(webFile.read()); // send web page to client
              }
              webFile.close();
            }
          }
          // display received HTTP request on serial port
          //          Serial.print(HTTP_req);
          // reset buffer index and all buffer elements to 0
          req_index = 0;
          StrClear(HTTP_req, REQ_BUF_SZ);
          break;
        }
        // every line of text received from the client ends with \r\n
        if (c == '\n') {
          // last character on line of received text
          // starting new line with next character read
          currentLineIsBlank = true;
          buffer="";  
        } 
        else if (c == '\r') {
          int switchNo = GetValue(buffer, "switch").toInt(); 
          int switchStatus = (GetValue(buffer, "status").toInt() == 0) ? LOW : HIGH;

          setRegisterPin(switchNo, switchStatus);
          // write the value to the appropriate byte of the EEPROM.
          // these values will remain there when the board is
          // turned off.
          //          EEPROM.update(switchNo, switchStatus);
          if( EEPROM.read(switchNo) != switchStatus ){
            EEPROM.write(switchNo, switchStatus);
          }
        } 

        else {
          // a text character was received from client
          currentLineIsBlank = false;
        }
      } // end if (client.available())
    } // end while (client.connected())
    delay(1);      // give the web browser time to receive the data
    client.stop(); // close the connection
  } // end if (client) 

  writeRegisters(); 
  allreley();
  //MUST BE CALLED TO DISPLAY CHANGES 
  //Only call once after the values are set how you need. 

  //  delay(1000);
}

// send the XML file containing analog value
void XML_response(EthernetClient cl)
{
  tempwReading = analogRead(tempw);
  float voltage = tempwReading * aref_voltage;
  voltage /= 1024.0; 
  float tempCW = (voltage - 0.5) * 100 ;  //converting from 10 mv per degree wit 500 mV offset

  float analog_val; //float anti int gia to koma
  int analog_val_2;
  float analog_val_3;
  int analog_val_4;
  int count;
  int digital_val;

  cl.print("<?xml version = \"1.0\" ?>");
  cl.print("<inputs>");

  analog_val = clock.readTemperature();
  cl.print("<analog>");
  cl.print(analog_val);
  cl.print("</analog>");
  analog_val_2 = analogRead(photocellPin);
  cl.print("<analog>");
  cl.print(analog_val_2);
  cl.print("</analog>");
  analog_val_3 = tempCW;
  cl.print("<analog>");
  cl.print(analog_val_3);
  cl.print("</analog>");
  analog_val_4 = floterReading;
  cl.print("<analog>");
  cl.print(analog_val_4);
  cl.print("</analog>");

  for (count = 0; count <= 8; count++) { 
    digital_val = EEPROM.read(count);
    cl.print("<digital>");
    cl.print(digital_val);
    cl.print("</digital>");

  }
    cl.print("</inputs>");  
}

// sets every element of str to 0 (clears array)
void StrClear(char *str, char length)
{
  for (int i = 0; i < length; i++) {
    str[i] = 0;
  }
}

// searches for the string sfind in the string str
// returns 1 if string found
// returns 0 if string not found
char StrContains(char *str, char *sfind)
{
  char found = 0;
  char index = 0;
  char len;
  len = strlen(str);
  if (strlen(sfind) > len) {
    return 0;
  }
  while (index < len) {
    if (str[index] == sfind[found]) {
      found++;
      if (strlen(sfind) == found) {
        return 1;
      }
    }
    else {
      found = 0;
    }
    index++;
  }
  return 0;
}


void logThis(char* logmessage){
  char message[120];
  dt = clock.getDateTime();
  long epoch = (dt.unixtime);
  int Year = (dt.year);
  int Month = (dt.month);
  int Day = (dt.day);
  int Hour = (dt.hour);
  int Minute = (dt.minute);
  int Second = (dt.second);
  sprintf(message, "%ld,%d/%d/%d %02d:%02d:%02d,%s",epoch,Year,Month,Day,Hour,Minute,Second,logmessage );
  //Write the entry to the log file and console
  dataFile.println(message);
  dataFile.flush();
  // print to the serial port too:
  Serial.println(message);

  //////////sd
  File dataFile = SD.open("datalog.txt", FILE_WRITE);
  if (dataFile) {
    dataFile.println(message);
    dataFile.close();
  }  
  // if the file isn't open, pop up an error:
  else {
    Serial.println("error opening datalog.txt");
  }
}
void checkAlarms()
{
  RTCAlarmTime a2;  


  if (clock.isArmed2())
  {
    a2 = clock.getAlarm2();

    Serial.print("Alarm2 is triggered ");
    switch (clock.getAlarmType2())
    {
    case DS3231_EVERY_SECOND:
      Serial.println("every second");
      break;
    case DS3231_MATCH_S:
      Serial.print("when seconds match: ");
      Serial.println(clock.dateFormat("__ __:__:s", a2));
      break;
    case DS3231_MATCH_M_S:
      Serial.print("when minutes and sencods match: ");
      Serial.println(clock.dateFormat("__ __:i:s", a2));
      break;
    case DS3231_MATCH_H_M_S:
      Serial.print("when hours, minutes and seconds match: ");
      Serial.println(clock.dateFormat("__ H:i:s", a2));
      break;
    case DS3231_MATCH_DT_H_M_S:
      Serial.print("when date, hours, minutes and seconds match: ");
      Serial.println(clock.dateFormat("d H:i:s", a2));
      break;
    case DS3231_MATCH_DY_H_M_S:
      Serial.print("when day of week, hours, minutes and seconds match: ");
      Serial.println(clock.dateFormat("l H:i:s", a2));
      break;
    default: 
      Serial.println("UNKNOWN RULE");
      break;
    }
  } 
  else
  {
    Serial.println("Alarm2 is disarmed.");
  }

}

void clearRegisters(){ 
  for(int i = numOfRegisterPins - 1; i >=  0; i--){ 
    registers[i] = LOW; 
  } 
} 


String GetValue(const String& str, const String& key){ 
  String params; 
  int qsPos = str.indexOf('?'); 
  if (qsPos < 0) return params; 

  params = str.substring(qsPos); 
  params.setCharAt(0,'&'); 
  String search('&'); 
  search += key; 
  search += '='; 
  qsPos = params.indexOf(search); 
  if (qsPos < 0) return ""; 

  qsPos += search.length(); 
  int qsPos2 = params.indexOf('&', qsPos+1); 
  if (qsPos2 < qsPos) 
  { 
    return params.substring(qsPos); 
  } 
  else 
  { 
    return params.substring(qsPos, qsPos2); 
  } 
} 
//Set and display registers 
//Only call AFTER all values are set how you would like (slow otherwise) 
void writeRegisters(){ 

  digitalWrite(RCLK_Pin, LOW); 

  for(int i = numOfRegisterPins - 1; i >=  0; i--){ 
    digitalWrite(SRCLK_Pin, LOW); 
    int val = registers[i]; 
    digitalWrite(SER_Pin, val);
    digitalWrite(SRCLK_Pin, HIGH); 
  } 
  digitalWrite(RCLK_Pin, HIGH); 
} 

//set an individual pin HIGH or LOW 
void setRegisterPin(int index, int value){ 
  registers[index] = value;
}
//off all relay /on apo eeprom
void allreley(){
  int off=0 ;
  int on=1;
  if( EEPROM.read(8) == off ){
    //reset all register pins 
    clearRegisters();
    writeRegisters();
    off=2;
    EEPROM.write(8, off);
  }
  else if ( EEPROM.read(8) == on ){
    setreley();
    on=3;
    EEPROM.write(8, on);
  }
} 

// set ssr to eeprom data meta apo diakopi revmatos
void setreley(){

  byte value;
  int count;
  for (count = 0; count <= 7; count++) { 
    value = EEPROM.read(count); 
    setRegisterPin(count, value);
  }
  writeRegisters() ;
 }



















